/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1998-1999 Jeremie Miller.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 * --------------------------------------------------------------------------*/
 
#include <jabberd.h>


/* 
gcc -fPIC -shared -o xdb_file.so xdb_file.c -I../src

needs to have a spool section in the config:

<service id="sms">
   <load><sms>./sms/sms.so</sms></load>
     <sms xmlns="jabber:config:sms">

     <host>localhost</host>
     <port>13013</port>
     <cgi>/cgi-bin/sendsms</cgi>
     <resource>sms</resource>
     <limitperuser>10</limitperuser>

     <paramname>
       <username/>
       <password/>
       <to/>
       <text/>
     </paramname>
  
     <paramvalue>
       <username>tester</username>
       <password>foobar</password>
     </paramvalue>
  
    </sms>

 </service>


http://localhost:13013/cgi-bin/sendsms?username=tester&password=foobar&to=0123456&text=Hello+world" ; done

*/

#define DEFAULT_LIMIT_PER_USER 10
#define DEFAULT_PARAM 23
#define DEFAULT_PORT 80


typedef struct _sms_param_struct
{
    char * translate;
    char * value;
} *smsparam, _smsparam;


typedef struct sms_struct
{
    HASHTABLE param;
    int limitperuser;
    char * host;
    int port;
    char * cgi;
    char * resource;
} *sms_data, _sms_data;



char *text2url(char * text){
  char * tmp;

  if (text == NULL)
    return NULL;

  for (tmp = text ; *tmp != '\0' ; tmp++){
    switch(*tmp){
    case ' ':
      *tmp = '+';
      break;
      
    default:
      
    }
  }
  
  return text;
}



result sms_phandler(instance i, dpacket p, void *arg){
    int fd;
    sms_data mysms = (sms_data) arg;
    char * to = NULL;
    char * buffer = NULL;
    xmlnode vcard = NULL;
    smsparam username, password, text;

    
    fprintf(stderr, "sms_handler: %s\n", xmlnode2str(p->x));
    fprintf(stderr, "p->id: %s\n", jid_full(p->id));

    if (p == NULL || p->x == NULL)
	return r_ERR;

    if (p->id->user){
      
	/*  Direct number to the instance  */
	to = p->id->user;
    }
    else{
	jid ujid;
	/*  jid/sms ... look in database for phone number  */
	ujid = jid_new(xmlnode_pool(p->x), xmlnode_get_attrib(p->x, "from"));
	
	jid_set(ujid, NULL, JID_RESOURCE);
	fprintf(stderr, "get vcard\n");
	fprintf(stderr, "from: %s\n",  xmlnode_get_attrib(p->x, "from"));
	vcard = xdb_get(xdb_cache(i), ujid, NS_VCARD);
	
	fprintf(stderr, "vcard: %s\n", xmlnode2str(vcard));
	
	if (!vcard)
	    return r_ERR;
	to = xmlnode_get_data(xmlnode_get_tag(vcard, "TEL"));	
    }


    username = (smsparam)(ghash_get(mysms->param, "username"));
    password = (smsparam)(ghash_get(mysms->param, "password"));
    text     = (smsparam)(ghash_get(mysms->param, "text"));

    buffer = spools(xmlnode_pool(p->x), "GET ", mysms->cgi, "?",
		    username->translate, "=", username->value, 
		    "&",
		    password->translate, "=", password->value,
		    "&",
		    "to=", to,
		    "&", 
		    text->translate, "=", 
		    "From+", xmlnode_get_attrib(p->x, "from"),
		    "++",
		    text2url(xmlnode_get_data(xmlnode_get_tag(p->x, "body"))),
		    " ", 
		    "HTTP/1.0",
		    "\r\n\n", xmlnode_pool(p->x));

    xmlnode_free(vcard);

    fd = make_netsocket(mysms->port, mysms->host, NETSOCKET_CLIENT);
    if (fd < 0)
	return r_ERR;
    

    fprintf(stderr, "sms : %s\n", buffer);
    

    {
	int res = 0;

	res = pth_write(fd, buffer, strlen(buffer));
	fprintf(stderr, "pth_write: res: %d  len: %d", res, strlen(buffer));
	
    }

    {
	char buf[256];
	int i = 0, res = 0;
	
	do{
	    res = pth_read(fd, &buf[i], 256);
	    fprintf (stderr, "res: %d\n", res);
	    i += res;
	}while (res > 0);

    }
    
     
    xmlnode_free(p->x);
    close(fd);  
    
    return r_DONE;
}


void sms(instance i, xmlnode x)
{
    xmlnode config, ptag = NULL;
    xdbcache xc;
    sms_data mysms;
    xmlnode ch;
    
    log_debug(ZONE,"dispatch loading");

    xc = xdb_cache(i);
    config = xdb_get(xc, jid_new(xmlnode_pool(x),"config@-internal"),"jabber:config:sms");
    
    mysms = (sms_data) pmalloco(i->p, sizeof(_sms_data));


    mysms->host = pstrdup(i->p, xmlnode_get_data(xmlnode_get_tag(config, "host")));
    mysms->resource = pstrdup(i->p, xmlnode_get_data(xmlnode_get_tag(config, "resource")));
    mysms->cgi = pstrdup(i->p, xmlnode_get_data(xmlnode_get_tag(config, "cgi")));
    mysms->port = j_atoi(xmlnode_get_data(xmlnode_get_tag(config, "port")), DEFAULT_PORT);
    mysms->limitperuser = j_atoi(xmlnode_get_data(xmlnode_get_tag(config, "limitperuser")), DEFAULT_LIMIT_PER_USER);
    mysms->param = ghash_create(DEFAULT_PARAM, (KEYHASHFUNC)str_hash_code, (KEYCOMPAREFUNC)j_strcmp);

    ptag = xmlnode_get_tag(config, "paramname");
    
    for(ch = xmlnode_get_firstchild(ptag); ch != NULL; ch = xmlnode_get_nextsibling(ch))
    {
	smsparam tmp;
	char * c;
	
	if (xmlnode_get_type(ch) != NTYPE_TAG)
	    continue;
	
	c =  pstrdup(i->p, xmlnode_get_name(ch));
	tmp = pmalloco(i->p, sizeof(_smsparam));
	if ( (tmp->translate = xmlnode_get_data(ch)) == NULL)
	    tmp->translate = c;
	    
	ghash_put(mysms->param, c,tmp);
    }


    ptag = xmlnode_get_tag(config, "paramvalue");

    for(ch = xmlnode_get_firstchild(ptag); ch != NULL; ch = xmlnode_get_nextsibling(ch))
    {
	smsparam tmp;
	
	if (xmlnode_get_type(ch) != NTYPE_TAG)
	    continue;
	
	tmp = (smsparam) ghash_get(mysms->param,  xmlnode_get_name(ch));
	tmp->value = pstrdup(i->p, xmlnode_get_data(ch));	    
    }
    
    

    register_phandler(i, o_DELIVER, sms_phandler, (void *)mysms);

    xmlnode_free(config);
//    pool_cleanup(i->p, xdb_file_cleanup, (void*)xf);
}




